@@ -219,7 +219,7 @@ makesockaddr(PySocketSockObject *s, struct sockaddr *addr, int addrlen)
                 {
                     struct sockaddr_l2 *a = (struct sockaddr_l2*)addr;
                     ba2str( &a->l2_bdaddr, ba_name );
-                    return Py_BuildValue("sH", ba_name, a->l2_psm );
+                    return Py_BuildValue("sH", ba_name, btohs(a->l2_psm) );
                 }
             case BTPROTO_RFCOMM:
                 {
@@ -277,6 +277,15 @@ getsockaddrarg(PySocketSockObject *s, PyObject *args,
                 }
 
                 str2ba( ba_name, &addr->l2_bdaddr );
+
+                // check for a valid PSM
+                if( ! ( 0x1 & addr->l2_psm ) ) {
+                    PyErr_SetString( PyExc_ValueError, "Invalid PSM");
+                    return 0;
+                }
+
+                addr->l2_psm = htobs(addr->l2_psm);
+
                 *len_ret = sizeof *addr;
                 return 1;
             }
@@ -1944,11 +1953,9 @@ bt_hci_inquiry(PyObject *self, PyObject *args, PyObject *kwds)
         Py_DECREF( list_entry );
         if (err) {
             Py_XDECREF( rtn_list );
-//            free(info);
             return NULL;
         }
     }
-//    free(info);
 
     return rtn_list;
 }
@@ -2421,7 +2428,13 @@ bt_sdp_advertise_service( PyObject *self, PyObject *args )
     // disconnect
     Py_BEGIN_ALLOW_THREADS
     session = sdp_connect( BDADDR_ANY, BDADDR_LOCAL, 0 );
+    Py_END_ALLOW_THREADS
+    if (!session) {
+        PyErr_SetFromErrno (bluetooth_error);
+        return 0;
+    }
     socko->sdp_session = session;
+    Py_BEGIN_ALLOW_THREADS
     err = sdp_record_register(session, &record, 0);
     Py_END_ALLOW_THREADS
 
